@@ -1,13 +1,13 @@
 #pca is just an alias for the principal function
 
-"pca" <- function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,scores=TRUE,missing=FALSE,impute="median",oblique.scores=TRUE,method="regression",...) {
-   principal(r=r,nfactors=nfactors,residuals=residuals,rotate=rotate,n.obs=n.obs,covar=covar,scores=scores, missing=missing, impute=impute,oblique.scores=oblique.scores, method=method,...)
+"pca" <- function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,scores=TRUE,missing=FALSE,impute="median",oblique.scores=TRUE,method="regression",use="pairwise",cor="cor",correct=.5,weight=NULL,...) {
+   principal(r=r,nfactors=nfactors,residuals=residuals,rotate=rotate,n.obs=n.obs,covar=covar,scores=scores, missing=missing, impute=impute,oblique.scores=oblique.scores, method=method,use=use,cor=cor,correct=.5,weight=NULL,...)
 
 }
 
 #to make principal components more similar in output to normal pca  in other systems.
 "principal" <-
-function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,scores=TRUE,missing=FALSE,impute="median",oblique.scores=TRUE,method="regression",...) {
+function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,scores=TRUE,missing=FALSE,impute="median",oblique.scores=TRUE,method="regression",use="pairwise",cor="cor",correct=.5,weight=NULL,...) {
    cl <- match.call()
    n <- dim(r)[2]
   
@@ -24,8 +24,27 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,s
        				item.med   <- apply(x.matrix,2,median,na.rm=TRUE) #replace missing with medians
        				 x.matrix[miss]<- item.med[miss[,2]]}
        				 }} 
-       				 # 2011.12.21  added the covar option
-      if(!covar) {r <- cor(r,use="pairwise")} else r <- cov(r,use="pairwise")  # if given a rectangular matrix, then find the correlations or covariances first
+       				 # 2011.12.21  added the covar option 
+       				 
+         switch(cor, 
+       cor = {if(!is.null(weight))  {r <- cor.wt(r,w=weight)$r} else  {
+                                     r <- cor(r,use=use)}
+                                     },
+       cov = {r <- cov(r,use=use) 
+              covar <- TRUE},
+       wtd = { r <- cor.wt(r,w=weight)$r},
+       spearman = {r <- cor(r,use=use,method="spearman")},
+       kendall = {r <- cor(r,use=use,method="kendall")},
+       tet = {r <- tetrachoric(r,correct=correct,weight=weight)$rho},
+       poly = {r <- polychoric(r,correct=correct,weight=weight)$rho},
+       tetrachoric = {r <- tetrachoric(r,correct=correct,weight=weight)$rho},
+       polychoric = {r <- polychoric(r,correct=correct,weight=weight)$rho},
+       mixed = {r <- mixedCor(r,use=use,correct=correct)$rho},
+       Yuleb = {r <- YuleCor(r,,bonett=TRUE)$rho},
+       YuleQ = {r <- YuleCor(r,1)$rho},
+       YuleY = {r <- YuleCor(r,.5)$rho } 
+       )				 
+   #   if(!covar) {r <- cor(r,use="pairwise")} else r <- cov(r,use="pairwise")  # if given a rectangular matrix, then find the correlations or covariances first
              } else {
                      raw <- FALSE
      				if(!is.matrix(r)) {  r <- as.matrix(r)}
